/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.DataParsers;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.Enumeration;
import java.util.Vector;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataCollection;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataSet;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.ProbeSet;

public class GenePixParser implements Runnable {

	private ChipMonkApplication application;
	private Vector listeners = new Vector();
	private ProbeSet probeSet;
	private DataCollection dataCollection;
	private DataSet [] dataSets;
	private File positions;
	private File [] dataFiles;
	
	
	public GenePixParser (ChipMonkApplication application) {
		this.application = application;
		dataCollection = application.dataCollection();
		probeSet = dataCollection.probeSet();
	}
	
	public ChipMonkApplication application () {
		return application;
	}
	
	public void parseFile (File positions, File [] dataFiles) {
	
		this.positions = positions;
		this.dataFiles = dataFiles;
		
		Thread t = new Thread(this);
		t.start();
	}
	
	public void addDataParserListener (DataParserListener l) {
		listeners.add(l);
	}
	
	public void run() {
		
		// Load the position data first
		int count = 0;
		try {
			BufferedReader br = new BufferedReader(new FileReader(positions));
			String line;
			while ((line = br.readLine()) != null) {
				count++;
				
				if ((count % 1000)==0) {
					Enumeration e = listeners.elements();
					while (e.hasMoreElements()) {
						((DataParserListener)e.nextElement()).dataUpdateProgress(0,1,"Loaded position data for "+count+" probes");
						try {
							Thread.sleep(5);
						}
						catch (InterruptedException ie){}
					}
				}
				if (line.length() == 0) continue;
				
				String [] sections = line.split("\\t");

				// Ignore short lines which are probably headers/comments
				if (sections.length < 3) continue;
				
				// Skip the header
				if (sections[0].equals("UHNID")) continue;
				if (sections[0].equals("")) continue;
				
				Probe p = application.dataCollection().probeSet().getProbeWithName(sections[0]);
				
				if (p == null) {

					// We need to make a new probe

					String chrName = null;
					int position = 0;
					
					String [] positions = sections[1].split("[:-]");
					if (positions.length == 3) {
						// We've got a real position
						chrName = positions[0].substring(3,positions[0].length());
						int startPos = Integer.parseInt(positions[1]);
						int endPos = Integer.parseInt(positions[2]);
						position = (startPos+endPos)/2;
					}

					try {
						p = new Probe(sections[0],application.genome().getChromosome(chrName),position);
					}
					catch (ChipMonkException ex) {
						ex.printStackTrace();
						continue;
					}
					probeSet.addProbe(p);
				}
			}
		}
		catch (Exception ex) {
			Enumeration e = listeners.elements();
			while (e.hasMoreElements()) {
				((DataParserListener)e.nextElement()).dataExceptionReceived(ex);
			}
			return;
		}
		
		
		// Now Parse the data files...
		try {
			String line;
			dataSets = new DataSet [dataFiles.length];
			for (int f=0;f<dataFiles.length;f++) {
				dataSets[f] = new DataSet(dataFiles[f].getName());
				count = 0;
				BufferedReader br = new BufferedReader(new FileReader(dataFiles[f]));
				while ((line = br.readLine()) != null) {
					count++;
					
					if ((count % 1000)==0) {
						Enumeration e = listeners.elements();
						while (e.hasMoreElements()) {
							((DataParserListener)e.nextElement()).dataUpdateProgress(f,dataFiles.length,"Loaded data for "+count+" probes in "+dataFiles[f].getName());
							try {
								Thread.sleep(5);
							}
							catch (InterruptedException ie){}
						}
					}
					if (line.length() == 0) continue;
					if (line.charAt(0)=='"') continue; // These are headers
					
					String [] sections = line.split("\\t");

					if (sections.length < 12) continue; // Not a data line

					Probe p = application.dataCollection().probeSet().getProbeWithName(sections[4].replaceAll("\"",""));
					
					if (p == null) {
						System.err.println("Couldn't find probe "+sections[4].replaceAll("\"","")+" in the probe set");
//						throw new ChipMonkException("Couldn't find probe "+sections[4].replaceAll("\"","")+" in the probe set");
					}
					
					dataSets[f].addData(p,Double.parseDouble(sections[9]),Double.parseDouble(sections[12]));
				}
			}
		}
		catch (Exception ex) {
			Enumeration e = listeners.elements();
			while (e.hasMoreElements()) {
				((DataParserListener)e.nextElement()).dataExceptionReceived(ex);
			}
			return;
		}
		
		for (int i=0;i<dataSets.length;i++) {
			dataCollection.addDataSet(dataSets[i]);
		}
		
		Enumeration e = listeners.elements();
		while (e.hasMoreElements()) {
			((DataParserListener)e.nextElement()).dataProcessingFinished(true);
		}
	}

}
