/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.DataParsers;

import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class NimblegenPairSelector extends JDialog implements ListSelectionListener, ActionListener {
	
	private int dataStartIndex;
	private DefaultListModel signals;
	private DefaultListModel controls;
	private DefaultListModel samples;
	private JList signalList;
	private JList controlList;
	private JList sampleList;
	private JButton makePairButton;
	private JButton finishedButton;
	private String [] headings;
	private boolean closedNicely = false;
	
	public NimblegenPairSelector (NimblegenParser parser, String [] headings, int dataStartIndex) {
		super(parser.application(),"Select Data Pairs");
		this.headings = headings;
		this.dataStartIndex = dataStartIndex;
		setModal(true);
				
		Container cp = getContentPane();
		cp.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		c.weighty = 0.1;
		c.fill = GridBagConstraints.HORIZONTAL;
		
		cp.add(new JLabel("Signal (Bound)",JLabel.CENTER),c);
		c.gridx++;
		cp.add(new JLabel("Control (Total)",JLabel.CENTER),c);
		c.gridx++;
		cp.add(new JLabel("Samples",JLabel.CENTER),c);

		c.gridx = 0;
		c.weighty = 10;
		c.gridy++;
		c.fill = GridBagConstraints.BOTH;
		
		signals = new DefaultListModel();
		for (int i=dataStartIndex;i<headings.length;i++) {
			signals.addElement(headings[i]);
		}
		
		signalList = new JList(signals);
		signalList.addListSelectionListener(this);
		signalList.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		cp.add(new JScrollPane(signalList),c);
		
		c.gridx++;
		
		controls = new DefaultListModel();
		for (int i=dataStartIndex;i<headings.length;i++) {
			controls.addElement(headings[i]);
		}
		
		controlList = new JList(controls);
		controlList.addListSelectionListener(this);
		controlList.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		cp.add(new JScrollPane(controlList),c);
		
		c.gridx++;
		
		samples = new DefaultListModel();
		
		sampleList = new JList(samples);
		cp.add(new JScrollPane(sampleList),c);
			
		JPanel buttonPanel = new JPanel();
		
		makePairButton = new JButton("Make new sample");
		makePairButton.setActionCommand("make_pair");
		makePairButton.addActionListener(this);
		makePairButton.setEnabled(false);
		buttonPanel.add(makePairButton);
		
		finishedButton = new JButton("Done");
		finishedButton.setActionCommand("done");
		finishedButton.addActionListener(this);
		finishedButton.setEnabled(false);
		buttonPanel.add(finishedButton);
		
		c.gridy++;
		c.gridx=0;
		c.gridwidth = 3;
		c.weighty = 0.1;
		c.fill = GridBagConstraints.HORIZONTAL;
		
		cp.add(buttonPanel,c);
		
		setSize(600,200);
		setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		setLocationRelativeTo(parser.application());
		setVisible(true);
		
	}

	public void valueChanged(ListSelectionEvent lse) {
		if (signalList.getSelectedIndex() >=0 && controlList.getSelectedIndex()>=0 && signalList.getSelectedIndex() != controlList.getSelectedIndex()) {
			makePairButton.setEnabled(true);
		}
		else {
			makePairButton.setEnabled(false);
		}
	}
	
	private int getHeadingIndex (String s) {
		for (int i=dataStartIndex;i<headings.length;i++) {
			if (headings[i].equals(s)) {
				return i;
			}
		}
		return -1;
	}

	public void actionPerformed(ActionEvent ae) {
		if (ae.getActionCommand().equals("make_pair")) {
			String signal = (String)signals.elementAt(signalList.getSelectedIndex());
			String control = (String)controls.elementAt(controlList.getSelectedIndex());
			String sampleName = ""; 
			
			while (sampleName.length() == 0) {
				sampleName = (String)JOptionPane.showInputDialog(this,"Enter sample name","Sample Name",JOptionPane.QUESTION_MESSAGE,null,null,signal);
				
				if (sampleName == null) {
					// They cancelled
					return;
				}
			
			}
			
			signals.removeElement(signal);
			controls.removeElement(signal);
			signals.removeElement(control);
			controls.removeElement(control);
						
			samples.addElement(new NimblegenSamplePair(sampleName,getHeadingIndex(signal),getHeadingIndex(control)));
			finishedButton.setEnabled(true);
		}
		else if (ae.getActionCommand().equals("done")) {
			closedNicely = true;
			setVisible(false);
		}
	}
	
	public NimblegenSamplePair [] getSamples () {
		if (closedNicely) {
			NimblegenSamplePair [] sampleArray = new NimblegenSamplePair [samples.size()];
			for (int i=0;i<samples.size();i++) {
				sampleArray[i] = (NimblegenSamplePair)samples.elementAt(i);
			}
		
			return sampleArray;
		}
		else {
			return null;
		}
	}
	
}
