/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Filters;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataGroup;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataSet;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.DataStore;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.ProbeList;
import uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Renderers.TypeColourRenderer;


public class DifferencesFilter extends JDialog implements ActionListener, KeyListener, ListSelectionListener, Runnable {

	private ChipMonkApplication application;
	private JList dataList;
	private JComboBox combineType;
	private JComboBox valueType;
	private JTextField lowerLimit;
	private JTextField upperLimit;
	private JButton okButton;
	private FilterProgressDialog progressDialog;
	private final static double log2 = Math.log(2);

	public DifferencesFilter (ChipMonkApplication application) {
		super(application,"Filter on values...");
		setSize(600,300);
		setLocationRelativeTo(application);
		
		this.application = application;

		getContentPane().setLayout(new BorderLayout());
		JPanel dataPanel = new JPanel();
		dataPanel.setBorder(BorderFactory.createEmptyBorder(4,4,0,4));
		dataPanel.setLayout(new BorderLayout());
		dataPanel.add(new JLabel("Data Sets/Groups",JLabel.CENTER),BorderLayout.NORTH);
		
		DefaultListModel dataModel = new DefaultListModel();
		
		DataSet [] sets = application.dataCollection().getAllDataSets();
		for (int i=0;i<sets.length;i++) {
			dataModel.addElement(sets[i]);
		}
		
		DataGroup [] groups = application.dataCollection().getAllDataGroups();
		for (int i=0;i<groups.length;i++) {
			dataModel.addElement(groups[i]);
		}
		
		
		dataList = new JList(dataModel);
		dataList.setCellRenderer(new TypeColourRenderer());
		dataList.addListSelectionListener(this);
		dataPanel.add(new JScrollPane(dataList),BorderLayout.CENTER);
		
		getContentPane().add(dataPanel,BorderLayout.WEST);
		
		JPanel choicePanel = new JPanel();
		choicePanel.setLayout(new BoxLayout(choicePanel,BoxLayout.Y_AXIS));
		
		JPanel choicePanel0 = new JPanel();
		JLabel probeListLabel = new JLabel ("Testing probes in '"+application.dataCollection().probeSet().getActiveProbeListName()+"' ("+application.dataCollection().probeSet().getActiveProbes().length+" probes)",JLabel.CENTER);
		probeListLabel.setFont(new Font("Default",Font.BOLD,12));
		choicePanel0.add(probeListLabel);
		choicePanel.add(choicePanel0);
		
		JPanel choicePanel1 = new JPanel();
		
		combineType = new JComboBox(new String [] {"Average","Maximum","Minimum"});
		combineType.setEnabled(false);
		choicePanel1.add(combineType);
		
		choicePanel1.add(new JLabel("difference in "));
		valueType = new JComboBox(new String [] {"Normalised log2 Ratio","Normalised Signal","Normalised Control","Raw Signal","Raw Control"});
		choicePanel1.add(valueType);
		choicePanel.add(choicePanel1);
		
		JPanel choicePanel2 = new JPanel();
		choicePanel2.add(new JLabel(" must be between "));
		lowerLimit = new JTextField(3);
		lowerLimit.addKeyListener(this);
		choicePanel2.add(lowerLimit);

		choicePanel2.add(new JLabel(" and "));
		
		upperLimit = new JTextField(3);
		upperLimit.addKeyListener(this);
		choicePanel2.add(upperLimit);
		choicePanel.add(choicePanel2);
				
		getContentPane().add(new JScrollPane(choicePanel),BorderLayout.CENTER);
		
		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Close");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		
		okButton = new JButton("Run Filter");
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		okButton.setEnabled(false);
		buttonPanel.add(okButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		setVisible(true);
	}

	private void makeProbeList () {
		
		// First things first, we need to check that everything we need
		// is ready for us.
		
		Object [] o = dataList.getSelectedValues();
		if (o.length <2) {
			JOptionPane.showMessageDialog(this,"At least 2 data sets must be selected selected","Error",JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		DataStore [] stores = new DataStore[o.length];
		for (int i=0;i<o.length;i++) {
			stores[i] = (DataStore)o[i];
		}
		
		// We'll convert the combo box options to letters so we
		// can use them in a switch statement later
		String v = (String)valueType.getSelectedItem();
		char selectedValue;
		if (v.equals("Normalised log2 Ratio"))
			selectedValue = 'R';
		else if (v.equals("Normalised Signal"))
			selectedValue = 'S';
		else if (v.equals("Normalised Control"))
			selectedValue = 'C';
		else if (v.equals("Raw Signal"))
			selectedValue = 's';
		else if (v.equals("Raw Control"))
			selectedValue = 'c';
		else {
			JOptionPane.showMessageDialog(this,"Didn't recognise value type "+v,"Error",JOptionPane.ERROR_MESSAGE);
			return;			
		}
		
		// Look at the numerical limits
		Double upper = null;
		Double lower = null;
		if (lowerLimit.getText().length()>0) {
			try {
				lower = new Double(Double.parseDouble(lowerLimit.getText()));
			}
			catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(this,"Lower limit was not a number","Error",JOptionPane.ERROR_MESSAGE);
				return;							
			}
		}

		if (upperLimit.getText().length()>0) {
			try {
				upper = new Double(Double.parseDouble(upperLimit.getText()));
			}
			catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(this,"Upper limit was not a number","Error",JOptionPane.ERROR_MESSAGE);
				return;							
			}
		}
		
		// Check we've got at least one limit
		if (lower == null && upper == null) {
			JOptionPane.showMessageDialog(this,"Neither a lower nor an upper limit was set","Error",JOptionPane.ERROR_MESSAGE);
			return;										
		}
		
		// If we've got two limits check the upper is bigger than the lower
		if (lower != null && upper != null) {
			if (lower.doubleValue() >= upper.doubleValue()) {
				JOptionPane.showMessageDialog(this,"Upper limit was not larger than the lower limit","Error",JOptionPane.ERROR_MESSAGE);
				return;							
			}
		}

		// Look at the limit type and convert to a char
		char selectedType;
		String l = (String)combineType.getSelectedItem();
		if (l.equals("Average"))
			selectedType = '=';
		else if (l.equals("Maximum"))
			selectedType = '>';
		else if (l.equals("Minumum"))
			selectedType = '<';
		else {
			JOptionPane.showMessageDialog(this,"Didn't recognise limit type "+l,"Error",JOptionPane.ERROR_MESSAGE);
			return;			
		}
						
		Probe [] probes = application.dataCollection().probeSet().getActiveProbes();
		ProbeList newList = new ProbeList(application.dataCollection().probeSet(),"Filtered Probes","Difference");
		
		for (int p=0;p<probes.length;p++) {
			
			if (p % 1000 == 0) {
				progressDialog.updateProgress(p, probes.length);
				try {
					Thread.sleep(10);
				} 
				catch (InterruptedException e1) {

				}
			}
			
			int count = 0;
			double d = 0;
			for (int s1=0;s1<stores.length;s1++) {
				for (int s2=s1+1;s2<stores.length;s2++) {
					switch (selectedType) {
					case '=':
						d+= getDifferenceValue(stores[s1],stores[s2],probes[p],selectedValue);
						count++;
						break;
					case '>':
						double dt1 = getDifferenceValue(stores[s1],stores[s2],probes[p],selectedValue);
						if (count == 0 || dt1 > d)
							d = dt1;
						count++;
						break;
					case '<':
						double dt2 = getDifferenceValue(stores[s1],stores[s2],probes[p],selectedValue);
						if (count == 0 || dt2 < d)
							d = dt2;
						count++;
						break;
					}
				}	
			}

			if (selectedType == '=') {
				d/=count;
			}
			
			// Now we have the value we need to know if it passes the test
			if (upper != null)
				if (d > upper.doubleValue())
					continue;
			
			if (lower != null)
				if (d < lower.doubleValue())
					continue;
			
			newList.addProbe(probes[p],new Double(d));
		}
		
		progressDialog.processingFinished();
		
		// See if any probes actually passed
		if (newList.getAllProbes().length == 0) {
			JOptionPane.showMessageDialog(this,"No probes matched the criteria set","Info",JOptionPane.INFORMATION_MESSAGE);
			return;
		}
		
		// Ask for a name for the list
		String groupName=null;
		while (true) {
			groupName = (String)JOptionPane.showInputDialog(this,"Enter list name","Found "+newList.getAllProbes().length+" probes",JOptionPane.QUESTION_MESSAGE,null,null,v+" diff between "+lower+"-"+upper);
			if (groupName == null)
				return;  // They cancelled
				
				
			if (groupName.length() == 0)
				continue; // Try again
			
			break;
		}
		newList.setName(groupName);
		
		
		
		try {
			application.dataCollection().probeSet().addProbeList(newList);
		} catch (ChipMonkException e) {
			e.printStackTrace();
		}
//		setVisible(false);
//		dispose();
		
	}
	
	private double getDifferenceValue (DataStore s1, DataStore s2, Probe p, char valueType) {
		double d=0;
		
		switch (valueType) {
		case 'R':
			d = (Math.log(s1.getRatioForProbe(p))/log2) - (Math.log(s2.getRatioForProbe(p))/log2);
			break;
		case 'S':
			d = s1.getSignalValueForProbe(p)-s2.getSignalValueForProbe(p);
			break;
		case 'C':
			d = s1.getControlValueForProbe(p)-s2.getControlValueForProbe(p);
			break;
		case 's':
			d = s1.getRawSignalValueForProbe(p)-s2.getRawSignalValueForProbe(p);
			break;
		case 'c':
			d = s1.getRawControlValueForProbe(p)-s2.getRawControlValueForProbe(p);
			break;
		}
		
		if (d<0) {
			d=0-d;
		}
		return d;
	}
	
	private void checkOK () {
		// Check to see if enough information has been added to allow us to
		// enable the OK button.
		
		// We have to have at least 2 datasets selected.
		if (dataList.getSelectedIndices().length < 2) {
			okButton.setEnabled(false);
			return;
		}
		
		// We need either an upper or a lower boundary
		if (lowerLimit.getText().length()==0 && upperLimit.getText().length()==0) {
			okButton.setEnabled(false);
			return;
		}
		
		
		// If we get here then we're good to go
		okButton.setEnabled(true);
		
	}
	
	public void actionPerformed(ActionEvent ae) {
		if (ae.getActionCommand().equals("cancel")) {
			setVisible(false);
			dispose();
		}
		else if (ae.getActionCommand().equals("ok")) {
			progressDialog = new FilterProgressDialog(this);
			Thread t = new Thread(this);
			t.start();
		}
	}

	public void keyTyped(KeyEvent arg0) {
	}

	public void keyPressed(KeyEvent ke) {

	}

	public void keyReleased(KeyEvent ke) {
		JTextField f = (JTextField)ke.getSource();
		
		char [] c = f.getText().toCharArray();
		
		StringBuffer b = new StringBuffer();
		for (int i=0;i<c.length;i++) {
			
			if (Character.isDigit(c[i])) {
				b.append(c[i]);
				continue;
			}
			if (c[i] == KeyEvent.VK_PERIOD) {
				b.append(c[i]);
				continue;
			}
			f.setText(b.toString());
			break;
		}
		
		
		checkOK();
	}

	public void valueChanged(ListSelectionEvent lse) {
		// If we have 2 or less items selected then we can disable the
		// combobox which says whether we're looking at min, max or average
		// difference (since they're all the same with only 1 comparison)
		if (dataList.getSelectedIndices().length<=2) {
			combineType.setEnabled(false);
		}
		else {
			combineType.setEnabled(true);
		}
		checkOK();
	}

	public void run() {
		makeProbeList();
	}
	
}
