/**
 * Copyright 2006-8 Simon Andrews
 *
 *    This file is part of ChIPMonk.
 *
 *    ChIPMonk is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    ChIPMonk is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with ChIPMonk; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.ChipMonk.Dialogs.Filters;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;

import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkApplication;
import uk.ac.bbsrc.babraham.ChipMonk.ChipMonkException;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Chromosome;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.Probe;
import uk.ac.bbsrc.babraham.ChipMonk.DataTypes.ProbeList;


public class PositionFilter extends JDialog implements ActionListener, KeyListener {

	private ChipMonkApplication application;
	private JComboBox chromosome;
	private JTextField start;
	private JTextField end;
	private JButton okButton;

	public PositionFilter (ChipMonkApplication application) {
		super(application,"Position filter...");
		setSize(400,200);
		setLocationRelativeTo(application);
		
		this.application = application;

		getContentPane().setLayout(new BorderLayout());
		
		JPanel choicePanel = new JPanel();

		choicePanel.setLayout(new GridBagLayout());
		
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx=0;
		gbc.gridy=0;
		gbc.weightx=0.2;
		gbc.weighty=0.5;
		gbc.fill=GridBagConstraints.HORIZONTAL;
		gbc.insets = new Insets(5,5,5,5);
		
		gbc.gridwidth=2;
		JLabel probeListLabel = new JLabel ("Starting with probes in '"+application.dataCollection().probeSet().getActiveProbeListName()+"' ("+application.dataCollection().probeSet().getActiveProbes().length+" probes)",JLabel.CENTER);
		probeListLabel.setFont(new Font("Default",Font.BOLD,12));
		choicePanel.add(probeListLabel,gbc);
		
		gbc.gridwidth=1;
		gbc.gridy++;
		
		choicePanel.add(new JLabel("Chromosome",JLabel.RIGHT),gbc);
	
		gbc.gridx++;
		gbc.weightx=0.6;
		
		Chromosome [] chrs = application.genome().getAllChromosomes();
		
		chromosome = new JComboBox(chrs);
		chromosome.setSelectedItem(application.chromosomeViewer().chromosome());
		
		choicePanel.add(chromosome,gbc);

		gbc.gridx=0;
		gbc.gridy++;
		gbc.weightx=0.2;
		
		choicePanel.add(new JLabel("From ",JLabel.RIGHT),gbc);
		
		gbc.gridx++;
		gbc.weightx=0.6;
		
		start = new JTextField(""+application.chromosomeViewer().currentStart(),5);
		start.addKeyListener(this);
		choicePanel.add(start,gbc);

		gbc.gridx=0;
		gbc.gridy++;
		gbc.weightx=0.2;
		
		choicePanel.add(new JLabel("To ",JLabel.RIGHT),gbc);
		
		gbc.gridx++;
		gbc.weightx=0.6;
		
		end = new JTextField(""+application.chromosomeViewer().currentEnd(),5);
		end.addKeyListener(this);
		choicePanel.add(end,gbc);

		
		getContentPane().add(new JScrollPane(choicePanel),BorderLayout.CENTER);
		
		JPanel buttonPanel = new JPanel();
		JButton cancelButton = new JButton("Close");
		cancelButton.setActionCommand("cancel");
		cancelButton.addActionListener(this);
		buttonPanel.add(cancelButton);
		
		okButton = new JButton("Run Filter");
		okButton.setActionCommand("ok");
		okButton.addActionListener(this);
		okButton.setEnabled(true);
		buttonPanel.add(okButton);
		
		getContentPane().add(buttonPanel,BorderLayout.SOUTH);
		
		setVisible(true);
	}

	private void makeProbeList () {
		
		Chromosome chr = (Chromosome)chromosome.getSelectedItem();
		int startValue = 1;
		int endValue = chr.length();

		if (start.getText().length()>0) {
			startValue = Integer.parseInt(start.getText());
		}
		if (end.getText().length() > 0) {
			endValue = Integer.parseInt(end.getText());
		}
		
		
		ProbeList newList = new ProbeList(application.dataCollection().probeSet(),"","No value");
		
		Probe [] probes = application.dataCollection().probeSet().getActiveProbesForChromosome(chr);
		
		for (int p=0;p<probes.length;p++) {
			if (probes[p].position()>=startValue && probes[p].position()<=endValue) {
				newList.addProbe(probes[p],new Double("NaN"));
			}
		}

		// See if any probes actually passed
		if (newList.getAllProbes().length == 0) {
			JOptionPane.showMessageDialog(this,"No probes matched the criteria set","Info",JOptionPane.INFORMATION_MESSAGE);
			return;
		}
		
		// Ask for a name for the list
		String groupName=null;
		while (true) {
			groupName = (String)JOptionPane.showInputDialog(this,"Enter list name","Found "+newList.getAllProbes().length+" probes",JOptionPane.QUESTION_MESSAGE,null,null,"Chr"+chr.name()+" "+startValue+"-"+endValue);
			if (groupName == null)
				return;  // They cancelled
				
				
			if (groupName.length() == 0)
				continue; // Try again
			
			break;
		}

		newList.setName(groupName);
		
		try {
			application.dataCollection().probeSet().addProbeList(newList);
		} catch (ChipMonkException e) {
			e.printStackTrace();
		}
		
//		setVisible(false);
//		dispose();
		
	}
	
	
	private void checkOK () {
		// Check to see if enough information has been added to allow us to
		// enable the OK button.
				
		// We need a start value
		
		// I've provided a default value so this is no longer needed
		
//		if (start.getText().length()==0) {
//			okButton.setEnabled(false);
//			return;
//		}

		// We need an end value

		// I've provided a default value so this is no longer needed

//		if (end.getText().length()==0) {
//			okButton.setEnabled(false);
//			return;
//		}

		// If we get here then we're good to go
		okButton.setEnabled(true);
		
	}
	
	public void actionPerformed(ActionEvent ae) {
		if (ae.getActionCommand().equals("cancel")) {
			setVisible(false);
			dispose();
		}
		else if (ae.getActionCommand().equals("ok")) {
			makeProbeList();
		}
	}

	public void keyTyped(KeyEvent arg0) {
	}

	public void keyPressed(KeyEvent ke) {

	}

	public void keyReleased(KeyEvent ke) {
		JTextField f = (JTextField)ke.getSource();
		
		char [] c = f.getText().toCharArray();
		
		StringBuffer b = new StringBuffer();

		for (int i=0;i<c.length;i++) {
				
			if (Character.isDigit(c[i])) {
				b.append(c[i]);
				continue;
			}
			f.setText(b.toString());
			break;
		}
				
		checkOK();
	}
}
